#include <reg52.h>
#include <stdio.h>
char keychoices[]="abc";



#define KEY_REPEAT_TIME 200  //set key press action
#define KEYALTERNATIVES 3
#define KEYSTOPPED 254

sbit BUTTONLED  =0xb6; //port3 6
sbit BUTTON   = 0xb7; //port3 7


volatile unsigned char keyTime;
char keyIndex=0;	
char key=0;


void readKey(void);
void serial_Init(void);
void sendChar(char c);


/*------------------------------------------------
Timer 0 Interrupt Service Routine.

------------------------------------------------*/

void timer0_ISR (void) interrupt 1
{
	 TL0=0;  //reload timer
	// TH0=0xdc;  //values give 10mS period
	 TH0=0xbc;  //longer time for Proteus simulation
	
	if (keyTime < KEY_REPEAT_TIME)   //hold at limit
	{
		keyTime++; 
	}
}



/*------------------------------------------------
MAIN C function
------------------------------------------------*/
void main (void)
{
/*--------------------------------------
Set Timer0 for 16-bit timer mode.  The
Set the Timer0 Run control bit.
--------------------------------------*/
TMOD = (TMOD & 0xF0) | 0x01;  /* Set T/C0 Mode */
	
ET0 = 1;                      /* Enable Timer 0 Interrupts */
TR0 = 1;                      /* Start Timer 0 Running */
EA = 1;                       /* Global Interrupt Enable */

	
 serial_Init();  //set serial port for output
	
/*--------------------------------------
The timer 0 interrupt will occur every 10mS
--------------------------------------*/
	
while (1)
  {
	    readKey();  //routine check
 
	if (keyTime ==KEY_REPEAT_TIME)  //counter stops at KEY_REPEAT_TIME
	{
		keyTime=KEYSTOPPED; //mark as this key done
		sendChar(0x0a); //new line
		sendChar(0x0d);
		sendChar(key); //final selection
		sendChar(0x0a); //new line
		sendChar(0x0d);
  }
	
}

}


void  readKey(void)
{
    if(	BUTTON)
		{
			return; //not pressed
		}

	//The key is pressed.. how long since last press
		
	if(keyTime < KEY_REPEAT_TIME)   //short time
	{
		keyIndex++;
		if(keyIndex >= KEYALTERNATIVES) keyIndex=0;	//0,1,2,0
	}
	else
	{
		keyIndex=0;	//long gap, so new key
	}

	//calculate key value
	
   key=	keychoices[keyIndex];
   sendChar(key);  // print current selection

	while(BUTTON==0){};  //wait for key to be released
	keyTime =0;  //reset the timer for next time
}


void Serial_Init()
{
	//Set timer 1 mode to 8-bit Auto-Reload
	TMOD=0x20;
	//Enable reception
	//Set Serial port mode to 8-bit UART
	SCON= 0x50;
	//Set baudrate to 9600 at 11.0592MHz
	 TH1=0xFD;
	TL1=0xFD;
	//Start Timer
	TR1=1;
	RI=0;  //clear recieved flag
	TI=1; //set all sent	
}
	 
void sendChar(char c)
{
     
	   while(TI==0){};  //wait until clear to send
			 TI=0; //reset
			 SBUF=c;
}